@@ -2,7 +2,7 @@
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
 #
-#      http://www.apache.org/licenses/LICENSE-2.0
+#      https://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
@@ -25,6 +25,8 @@ PATCH = 'PATCH'
 POST = 'POST'
 PUT = 'PUT'
 
+_original_send = requests.Session.send
+
 
 class MockerCore(object):
     """A wrapper around common mocking functions.
@@ -33,11 +35,14 @@ class MockerCore(object):
     same general options available and prevent repeating code.
     """
 
-    _PROXY_FUNCS = set(['last_request',
-                        'add_matcher',
-                        'request_history',
-                        'called',
-                        'call_count'])
+    _PROXY_FUNCS = {
+        'last_request',
+        'add_matcher',
+        'request_history',
+        'called',
+        'called_once',
+        'call_count',
+    }
 
     case_sensitive = False
     """case_sensitive handles a backwards incompatible bug. The URL used to
@@ -51,6 +56,11 @@ class MockerCore(object):
 
     requests_mock.mock.case_sensitive = True
 
+    or for pytest set in your configuration:
+
+    [pytest]
+    requests_mock_case_sensitive = True
+
     which will prevent the lowercase being executed and return case sensitive
     url and query information.
 
@@ -58,11 +68,14 @@ class MockerCore(object):
     """
 
     def __init__(self, **kwargs):
-        case_sensitive = kwargs.pop('case_sensitive', self.case_sensitive)
-        self._adapter = adapter.Adapter(case_sensitive=case_sensitive)
+        self.case_sensitive = kwargs.pop('case_sensitive', self.case_sensitive)
+        self._adapter = (
+            kwargs.pop('adapter', None) or
+            adapter.Adapter(case_sensitive=self.case_sensitive)
+        )
 
         self._real_http = kwargs.pop('real_http', False)
-        self._real_send = None
+        self._last_send = None
 
         if kwargs:
             raise TypeError('Unexpected Arguments: %s' % ', '.join(kwargs))
@@ -72,10 +85,10 @@ class MockerCore(object):
 
         Install the adapter and the wrappers required to intercept requests.
         """
-        if self._real_send:
+        if self._last_send:
             raise RuntimeError('Mocker has already been started')
 
-        self._real_send = requests.Session.send
+        self._last_send = requests.Session.send
 
         def _fake_get_adapter(session, url):
             return self._adapter
@@ -84,8 +97,18 @@ class MockerCore(object):
             real_get_adapter = requests.Session.get_adapter
             requests.Session.get_adapter = _fake_get_adapter
 
+            # NOTE(jamielennox): self._last_send vs _original_send. Whilst it
+            # seems like here we would use _last_send there is the possibility
+            # that the user has messed up and is somehow nesting their mockers.
+            # If we call last_send at this point then we end up calling this
+            # function again and the outer level adapter ends up winning.
+            # All we really care about here is that our adapter is in place
+            # before calling send so we always jump directly to the real
+            # function so that our most recently patched send call ends up
+            # putting in the most recent adapter. It feels funny, but it works.
+
             try:
-                return self._real_send(session, request, **kwargs)
+                return _original_send(session, request, **kwargs)
             except exceptions.NoMockAddress:
                 if not self._real_http:
                     raise
@@ -96,7 +119,7 @@ class MockerCore(object):
             finally:
                 requests.Session.get_adapter = real_get_adapter
 
-            return self._real_send(session, request, **kwargs)
+            return _original_send(session, request, **kwargs)
 
         requests.Session.send = _fake_send
 
@@ -105,9 +128,9 @@ class MockerCore(object):
 
         This should have no impact if mocking has not been started.
         """
-        if self._real_send:
-            requests.Session.send = self._real_send
-            self._real_send = None
+        if self._last_send:
+            requests.Session.send = self._last_send
+            self._last_send = None
 
     def __getattr__(self, name):
         if name in self._PROXY_FUNCS:
@@ -185,7 +208,8 @@ class Mocker(MockerCore):
         """
         m = Mocker(
             kw=self._kw,
-            real_http=self._real_http
+            real_http=self._real_http,
+            case_sensitive=self.case_sensitive
         )
         return m
 
@@ -196,7 +220,7 @@ class Mocker(MockerCore):
         """
         @functools.wraps(func)
         def inner(*args, **kwargs):
-            with self as m:
+            with self.copy() as m:
                 if self._kw:
                     kwargs[self._kw] = m
                 else:
