@@ -2,7 +2,7 @@
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
 #
-#      http://www.apache.org/licenses/LICENSE-2.0
+#      https://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
@@ -17,6 +17,7 @@ from requests.cookies import MockRequest, MockResponse
 from requests.cookies import RequestsCookieJar
 from requests.cookies import merge_cookies, cookiejar_from_dict
 from requests.packages.urllib3.response import HTTPResponse
+from requests.utils import get_encoding_from_headers
 import six
 
 from requests_mock import compat
@@ -145,23 +146,24 @@ def create_response(request, **kwargs):
     content = kwargs.pop('content', None)
     text = kwargs.pop('text', None)
     json = kwargs.pop('json', None)
+    headers = kwargs.pop('headers', {})
     encoding = None
 
-    if content and not isinstance(content, six.binary_type):
-        raise TypeError('Content should be a callback or binary data')
-    if text and not isinstance(text, six.string_types):
-        raise TypeError('Text should be a callback or string data')
+    if content is not None and not isinstance(content, six.binary_type):
+        raise TypeError('Content should be binary data')
+    if text is not None and not isinstance(text, six.string_types):
+        raise TypeError('Text should be string data')
 
     if json is not None:
         text = jsonutils.dumps(json)
     if text is not None:
-        encoding = 'utf-8'
+        encoding = get_encoding_from_headers(headers) or 'utf-8'
         content = text.encode(encoding)
     if content is not None:
         body = _IOReader(content)
     if not raw:
         raw = HTTPResponse(status=kwargs.get('status_code', _DEFAULT_STATUS),
-                           headers=kwargs.get('headers', {}),
+                           headers=headers,
                            reason=kwargs.get('reason'),
                            body=body or _IOReader(six.b('')),
                            decode_content=False,
@@ -170,7 +172,9 @@ def create_response(request, **kwargs):
 
     response = _http_adapter.build_response(request, raw)
     response.connection = connection
-    response.encoding = encoding
+
+    if encoding and not response.encoding:
+        response.encoding = encoding
 
     _extract_cookies(request, response, kwargs.get('cookies'))
 
@@ -207,12 +211,12 @@ class _MatcherResponse(object):
         content = self._params.get('content')
         text = self._params.get('text')
 
-        if content and not (callable(content) or
-                            isinstance(content, six.binary_type)):
+        if content is not None and not (callable(content) or
+                                        isinstance(content, six.binary_type)):
             raise TypeError('Content should be a callback or binary data')
 
-        if text and not (callable(text) or
-                         isinstance(text, six.string_types)):
+        if text is not None and not (callable(text) or
+                                     isinstance(text, six.string_types)):
             raise TypeError('Text should be a callback or string data')
 
     def get_response(self, request):
